import * as React from "react";
import { Bounds } from "./Hooks/use-measure";
import { CSSProperties } from "react";
import { InputSize } from "./Form";
/**
 * Combobox context
 */
interface ComboBoxContextType {
    /** element refs */
    inputRef: React.RefObject<HTMLInputElement>;
    listRef: React.RefObject<HTMLElement>;
    /** list options */
    options: React.MutableRefObject<string[] | null>;
    makeHash: (i: string) => string;
    selected: string | null;
    expanded: boolean;
    /** event handlers */
    onInputChange: (e: React.ChangeEvent<HTMLInputElement>) => void;
    handleBlur: () => void;
    handleFocus: () => void;
    handleOptionSelect: (value: string) => void;
    onKeyDown: (e: React.KeyboardEvent) => void;
    /** popover positions */
    position: any;
    inputSize: Bounds;
    listId: string;
    query: string | number;
    onInputCommit: (text: string) => void;
}
export declare const ComboBoxContext: React.Context<ComboBoxContextType>;
/**
 * Context provider / manager
 * 手动添加的 style?: CSSProperties;
 */
export interface ComboBoxProps {
    style?: CSSProperties;
    onSelect?: (selected: string) => void;
    query: string | number;
    onQueryChange: (value: string | number) => void;
    onInputCommit: (value: string | number) => void;
    children?: React.ReactNode;
}
/** 请用ComboBoxDatalist来代替 ComboBox；
 * 虽然直接用W3C浏览器提供的<datalist标签？list做关联id,来做组合输入框，请到Form.tsx找InputDatalist组件代替本组件，
 * 但是还是这个定制的更好用。
 * ComboBox是内部使用 private的。
 * 父辈组件注入style样式，然后传递给孙子组件ComboBoxInput。
 遇到嵌套组件<ComboBox> <ComboBoxInput； 照样能够把style传递给孙子儿子ComboBoxInput；
 */
export declare const ComboBox: React.FunctionComponent<ComboBoxProps>;
/**
 * Input element
 */
export interface ComboBoxInputProps extends React.HTMLAttributes<any> {
    "aria-label": string;
    component?: React.ElementType<any>;
    [key: string]: any;
}
export declare const ComboBoxInput: React.FunctionComponent<ComboBoxInputProps>;
/**
 * Popover container
 */
export interface ComboBoxListProps extends React.HTMLAttributes<HTMLUListElement> {
    autoHide?: boolean;
}
export declare const ComboBoxList: React.FunctionComponent<ComboBoxListProps>;
/**
 * Individual combo box options
 */
export interface ComboBoxOptionProps extends React.HTMLAttributes<HTMLLIElement> {
    value: string;
}
/** ComboBox不允许列表派生参数label，字面描述就是value。
 * value才是选择输出数值， 但是ComboBoxOption组件可以内嵌子组件来表达诸如表面描述或者图片等与value有关联的东西label。
 * 但是选择了之后，在Input框显示的还是value，不能显示出label表面描述文字。
 * 类似这样用户可以修改的输入，应该不会在意关联的label表面描述文字，真的应该去掉，仅仅保留value；不像Select这样的固定的/用户无法主动输入只允许选择的才会搞出label名堂。
 * @param value
 * @param children
 * @param other
 * ComboBox不允许multiple；但是Select不允许自主添加新的列表项目，Select允许列表附加label描述文字代替value;
 */
export declare const ComboBoxOption: React.FunctionComponent<ComboBoxOptionProps>;
/**
 * ComboBox Item text with highlighting
 */
export interface ComboBoxOptionTextProps {
    value: string;
}
export declare const ComboBoxOptionText: React.FunctionComponent<ComboBoxOptionTextProps>;
export interface ComboBoxDatalistProps extends React.InputHTMLAttributes<HTMLInputElement> {
    /** The size of the input element */
    inputSize?: InputSize;
    /** 控制是否满上宽度;
     *  需要在<input> 上 去控制大尺寸上限的width:  ，以及自适应屏幕大小后的 max-width: 缩小尺寸。
     * */
    fullWidth?: boolean;
    datalist?: any[];
    onListChange: (value: string | number) => void;
    value?: string | number;
}
/**类似InputDatalist组件，目的是：可以直接替换 repalce in files，能保持参数一致，移植修改简单切换。
 * 对比InputDatalist,定做组合框的版本。 而且InputDatalist弹出列表无法自己定做样式。
 * 自主输入Enter完成就变空的。 ComboBoxInput无法使用safeBind({ ref }，单独有inputRef的；
 * readOnly?: boolean; disabled?: boolean;
 * onChange不能直接利用参数类型是event：e?.currentTarget，而ComboBox这都是string类型。
 * @param onListChange:可选列表选定，或者编辑框修改,导致query={value}必然变更。
 */
export declare const ComboBoxDatalist: React.FunctionComponent<ComboBoxDatalistProps>;
export {};
