import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import React from 'react';
import { Global, css } from "@emotion/react";
// import * as React from "react";
// import toaster, { Position } from "react-toastnotes";
import { AlertContent, } from "./Alert";
import { useTheme, ThemeProvider } from "./Theme/Providers";
import { ToastContainer, toast } from 'react-toastify';
import 'react-toastify/dist/ReactToastify.css';
//这个是针对ReactToastify.css 内部消息父辈div的样式 .Toaster__message-wrapper {padding: 8px;}
const toastStyles = css `
  .Toastify__toast-container {
        //width: var(--toastify-toast-width);
    width: fit-content
  }
`;
/** 类Hook的 需要用到jsx, css的场合,不同一般地Hook。
 * We export toast as a hook because it allows us to consume
 * the current theme context that it's in and pass that
 * onto our render function
 报错来源： ReactDOM.render is no longer supported in React 18. Use createRoot instead. Until you switch to the new API, your app will behave as if it's running React 17.
 【配套】 必须在APP顶级组件中 再配个<ToastContainer />才能生效。
 */
export function useToast() {
    const theme = useTheme();
    function notify({ position = 'bottom-center', duration, render, title, subtitle, intent }) {
        //适配旧的包接口函数参数
        const options = {
            position,
            autoClose: duration,
            type: intent,
        };
        //宽度已经被上一级的 react-toastify包 /ReactToastify.css 文件 --toastify-toast-width: 320px;所限定了最大宽度！！
        //俩条路都是 （return React.ReactNode）纯字符创也算是: React.ReactNode;
        if (render) {
            return toast(({ closeToast: onClose, toastProps: { toastId: id } }) => (_jsxs(React.Fragment, { children: [_jsx(Global, { styles: toastStyles }), _jsx(ThemeProvider, { theme: theme, children: render({ onClose, id }) })] })), options);
        }
        //普通的 export type MessageProp = React.ReactNode | ((callback: MessageCallback) => React.ReactNode) | string;
        toast(({ closeToast: onClose, toastProps: { toastId: id } }) => (_jsxs(React.Fragment, { children: [_jsx(Global, { styles: toastStyles }), _jsx(ThemeProvider, { theme: theme, children: _jsx(AlertContent, { id: String(id), title: title, subtitle: subtitle }) })] })), options);
    }
    //【无法显示】必须配套 在APP顶级组件中 再加个<ToastContainer />才能显示。
    return notify;
}
export { ToastContainer };
//# sourceMappingURL=Toast.js.map