import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import { css } from "@emotion/react";
import * as React from "react";
import { useContext, } from "react";
import { TabPanel } from "./Tabs";
import { Pager } from "./Pager";
import { useMMenuBarOcup } from "./MainMenuBar";
import { useTheme } from "./Theme/Providers";
import { Layer } from "./Layer";
/** 右半边详细操作页面被放大为整个四口的页面，放大
 * 跨越多层次的路由器页面。
 * */
export const PanelEnlargeCtx = React.createContext({
    enlarge: undefined,
    setEnlarge: value => null,
});
/**公用框架组件  Left左半列表摘要 + 右半边Console详细操作的页面。
 * 还允许把状态 activeTab, setActiveTab] = React.useState( 提升到外部注入，方便外来自定义控制逻辑。
 * 本组件是框架，要配合 <TwoHalfRightPanel/>组件 以及<PanelEnlargeCtx.Provider 这两个东西来协同完成框架页面构建。
 * 需要在页面路由器头上加<PanelEnlargeCtx.Provider  value={{ enlarge, setEnlarge }}>
 * */
export const TwoHalfFrame = ({ children, rightPanel, activeTab, setActiveTab, onPagerTermination, leftcsStyle, leftncStyle, rightcsStyle, rightncStyle, ...other }) => {
    const theme = useTheme();
    const { enlarge, } = useContext(PanelEnlargeCtx);
    const { renderList, bottomCss, carousel } = useMMenuBarOcup(!rightPanel, enlarge);
    //针对非触摸手机的宽度够大的屏幕，两个半页面拼凑的模式的： 屏幕宽度相当于电脑的触摸屏也是走这个规则的。
    const notCarousel = (_jsxs(React.Fragment, { children: [_jsx(Layer, { className: "LeftPanelS", "aria-hidden": !renderList, css: [
                    {
                        display: renderList ? "flex" : "none",
                        boxSizing: "border-box",
                        flexDirection: "column",
                        flex: "1",
                        boxShadow: "none",
                        background: "white",
                        position: "fixed",
                        width: "100%",
                        height: "100%",
                        borderRadius: 0,
                        margin: 0,
                        overflowY: "auto",
                        [theme.mediaQueries.md]: {
                            display: "flex",
                            zIndex: enlarge ? 'unset' : theme.zIndices.fixed - 1,
                            top: 0,
                            boxShadow: theme.shadows.xs,
                            width: "50%",
                            // borderRadius: theme.radii.lg,
                        },
                    },
                    { ...leftncStyle },
                ], children: children }), rightPanel && (_jsx(Layer, { className: "RightPanel", elevation: "xl", css: [
                    {
                        display: 'flex',
                        alignItems: 'center',
                        borderRadius: 0,
                        position: "relative",
                        boxShadow: "none",
                        // width: enlarge? "100%" : "50%",
                        width: renderList && !enlarge ? "50%" : "100%",
                        height: '100vh',
                        background: 'transparent',
                        [theme.mediaQueries.md]: {
                            marginTop: "auto",
                            marginBottom: "auto",
                            borderRadius: theme.radii.lg,
                            boxShadow: theme.shadows.xl,
                            paddingLeft: enlarge ? 'unset' : "50%", //排除掉左边的半边的宽度
                        }
                    },
                    // renderList&&!enlarge? undefined : bottomCss,
                ], children: _jsx("div", { className: "RightPanelMiddle", css: [
                        {
                            display: "block",
                            //position: "absolute",
                            width: "100%",
                            height: "max-content",
                            background: 'white',
                            top: 0,
                            bottom: 0,
                            margin: 'auto',
                            minHeight: "50vh",
                            //boxSizing: "border-box",
                            [theme.mediaQueries.md]: {
                                justifyContent: "center",
                                alignItems: "center",
                                // paddingLeft: enlarge? 'unset' : "50%",     //排除掉左边的半边的宽度
                            },
                        },
                        { ...rightncStyle },
                    ], children: rightPanel }) }, 1))] }));
    //主体复合框架： 手机 + 电脑的。手机对sticky height:100vh 100%处理不同于电脑模拟器。
    return (_jsxs("div", { css: css ` box-sizing: border-box; `, ...other, children: [carousel &&
                _jsxs(Pager, { className: "TouchPhone", lazyLoad: false, enableScrollLock: true, value: activeTab, onRequestChange: i => setActiveTab(i), enableMouse: true, minVelocity: 0.2, onTerminationRequest: onPagerTermination, css: {
                        height: '100vh',
                        overflow: 'hidden',
                        //overflowY: "auto",
                    }, paneStyle: {
                    //overflowX: 'hidden'
                    }, children: [_jsx(TabPanel, { id: "ListSummary", css: {
                                ...leftcsStyle
                            }, children: children }), _jsx(TabPanel, { id: "DetailsEditor", css: {
                                ...rightcsStyle
                            }, children: rightPanel })] }), !carousel && (notCarousel)] }));
};
//# sourceMappingURL=TwoHalfFrame.js.map